from __future__ import annotations

from typing import Any
from typing import Callable
from typing import List
from typing import TYPE_CHECKING
from typing import TypeVar
from typing import Union


if TYPE_CHECKING:
    from setuptools_scm import version
    import os

from typing_extensions import ParamSpec, TypeAlias, Protocol

PathT = Union["os.PathLike[str]", str]

CMD_TYPE: TypeAlias = Union[List[str], str]

VERSION_SCHEME = Union[str, Callable[["version.ScmVersion"], str]]


class EntrypointProtocol(Protocol):
    name: str

    def load(self) -> Any:
        pass


T = TypeVar("T")
T2 = TypeVar("T2")
P = ParamSpec("P")


def transfer_input_args(
    template: Callable[P, T],
) -> Callable[[Callable[..., T]], Callable[P, T]]:
    def decorate(func: Callable[..., T2]) -> Callable[P, T2]:
        return func

    return decorate



from __future__ import annotations

import os
from typing import IO


def read_dist_name_from_setup_cfg(
    input: str | os.PathLike[str] | IO[str] = "setup.cfg",
) -> str | None:

    # minimal effort to read dist_name off setup.cfg metadata
    import configparser

    parser = configparser.ConfigParser()

    if isinstance(input, (os.PathLike, str)):
        parser.read([input])
    else:
        parser.read_file(input)

    dist_name = parser.get("metadata", "name", fallback=None)
    return dist_name
